@@ -60,8 +60,7 @@ keywords are treated in a special way:
  'target' -- If any template method like execute finds a 'target' keyword in
      the message then this is used as the target value in the return value.
      However, if _execute then also returns a target value this overwrites the
-     target value. In global_message calls 'target' has no special meaning and
-     can be used like any other keyword.
+     target value.
 
   'method' -- Specify the name of the method that should be used instead of
       the standard one (e.g. in execute the standard method is _execute).
@@ -71,6 +70,8 @@ keywords are treated in a special way:
       and if no target is provided it will be set to -1.
 
 """
+from builtins import next
+from builtins import str
 
 import inspect
 
@@ -207,7 +208,7 @@ class BiNode(mdp.Node):
                     self._check_train_args(x, **arg_dict)
                 except TypeError:
                     err = ("The given additional arguments %s " %
-                                str(arg_dict.keys()) +
+                                str(list(arg_dict.keys())) +
                            "are not compatible with training %s." % str(self))
                     raise BiNodeException(err)
                 self._train_phase_started = True
@@ -520,7 +521,7 @@ def binode_coroutine(args=None, defaults=()):
         def _coroutine_interface(self, *args):
             try:
                 return self._coroutine_instances[original_name].send(args)
-            except StopIteration, exception:
+            except StopIteration as exception:
                 delattr(self, original_name)
                 del self._coroutine_instances[original_name]
                 if len(exception.args):
@@ -543,8 +544,8 @@ def binode_coroutine(args=None, defaults=()):
             self._coroutine_instances[original_name] = coroutine_instance
             setattr(self, original_name, bound_coroutine_interface)
             try:
-                return coroutine_instance.next()
-            except StopIteration, exception:
+                return next(coroutine_instance)
+            except StopIteration as exception:
                 delattr(self, original_name)
                 del self._coroutine_instances[original_name]
                 if len(exception.args):
