@@ -1,12 +1,14 @@
+from builtins import range
+from builtins import object
 import py.test
 
 import mdp
 from mdp import numx as np
 
 from bimdp import (
-    MessageResultContainer, BiFlow, BiFlowException, EXIT_TARGET, nodes
+    MessageResultContainer, BiFlow, BiFlowException, EXIT_TARGET, nodes,
 )
-from _tools import TraceJumpBiNode, IdNode
+from ._tools import TraceJumpBiNode, IdNode
 
 
 class TestMessageResultContainer(object):
@@ -70,6 +72,15 @@ class TestBiFlow(object):
         x = np.random.random([100,10])
         flow.execute(x)
 
+    def test_index_with_node_ids(self):
+        """Test a BiFlow indexed by keys."""
+        pca_node = nodes.PCABiNode(node_id="pca")
+        biflow = BiFlow([pca_node])
+        x = biflow["pca"]
+        assert x is pca_node
+        assert pca_node in biflow
+        assert 'pca' in biflow
+
     def test_normal_multiphase(self):
         """Test training and execution with multiple training phases.
 
