@@ -1,4 +1,4 @@
-from _tools import *
+from ._tools import *
 
 def testHistogramNode_nofraction():
     """Test HistogramNode with fraction set to 1.0."""
