@@ -1,5 +1,5 @@
 import py.test
-from _tools import *
+from ._tools import *
 
 
 class DummyPreserveDimNode(mdp.PreserveDimNode):
