@@ -1,11 +1,13 @@
-from __future__ import with_statement
+from future import standard_library
+standard_library.install_aliases()
+from builtins import range
+from builtins import object
 
 import tempfile
 import pickle
-import cPickle
+import pickle
 import os
-import inspect
-from _tools import *
+from ._tools import *
 
 uniform = numx_rand.random
 
@@ -26,7 +28,7 @@ class _CheckpointCollectFunction(mdp.CheckpointFunction):
 def testFlow():
     inp = numx.ones((100,3))
     flow = _get_default_flow()
-    for i in xrange(len(flow)):
+    for i in range(len(flow)):
         assert not flow.flow[i].is_training(), \
                'Training of node #%d has not been closed.' % i
 
@@ -50,7 +52,7 @@ def test_Flow_copy_with_lambda():
     generic_node = mdp.Node()
     generic_node.lambda_function = lambda: 1
     generic_flow = mdp.Flow([generic_node])
-    copy_flow = generic_flow.copy()
+    generic_flow.copy()
     
 def testFlow_save():
     dummy_list = [1,2,3]
@@ -58,18 +60,18 @@ def testFlow_save():
     flow[0].dummy_attr = dummy_list
     # test string save
     copy_flow_pic = flow.save(None)
-    copy_flow = cPickle.loads(copy_flow_pic)
+    copy_flow = pickle.loads(copy_flow_pic)
     assert flow[0].dummy_attr == copy_flow[0].dummy_attr, \
            'Flow save (string) method did not work'
     copy_flow[0].dummy_attr[0] = 10
     assert flow[0].dummy_attr != copy_flow[0].dummy_attr, \
            'Flow save (string) method did not work'
     # test file save
-    dummy_file = os.path.join(tempfile.gettempdir(),'removeme')
+    dummy_file = tempfile.mktemp(prefix='MDP_', suffix=".pic",
+                                 dir=py.test.mdp_tempdirname)
     flow.save(dummy_file, protocol=1)
-    with open(dummy_file, 'rb') as flh:
-        copy_flow = cPickle.load(flh)
-    os.remove(dummy_file)
+    dummy_file = open(dummy_file, 'rb')
+    copy_flow = pickle.load(dummy_file)
     assert flow[0].dummy_attr == copy_flow[0].dummy_attr, \
            'Flow save (file) method did not work'
     copy_flow[0].dummy_attr[0] = 10
@@ -82,7 +84,7 @@ def testFlow_container_privmethods():
     # test __len__
     assert_equal(len(flow), len(flow.flow))
     # test __?etitem__, integer key
-    for i in xrange(len(flow)):
+    for i in range(len(flow)):
         assert flow[i]==flow.flow[i], \
                '__getitem__  returned wrong node %d' % i
         new_node = BogusNode()
@@ -111,7 +113,7 @@ def testFlow_container_privmethods():
     copy_flow = mdp.Flow(flow[:])
     del copy_flow[0]
     assert len(copy_flow) == len(flow)-1, '__delitem__ did not del'
-    for i in xrange(len(copy_flow)):
+    for i in range(len(copy_flow)):
         assert copy_flow[i] == flow[i+1], '__delitem__ deleted wrong node'
     # test __delitem__, normal slice
     copy_flow = mdp.Flow(flow[:])
@@ -148,7 +150,7 @@ def testFlow_container_listmethods():
     try:
         newnode = BogusNode(input_dim=11)
         flow.append(newnode)
-        raise Exception, 'flow.append appended inconsistent node'
+        raise Exception('flow.append appended inconsistent node')
     except ValueError:
         assert_equal(len(flow), length)
     # extend
@@ -158,14 +160,14 @@ def testFlow_container_listmethods():
     length = len(flow)
     try:
         newflow = _get_default_flow()
-        for idx in xrange(len(newflow)):
+        for idx in range(len(newflow)):
             if idx == 0:
                 newflow[idx].input_dim = 11
             else:
                 newflow[idx].input_dim = 10
             newflow[idx].output_dim = 10
         flow.extend(newflow)
-        raise Exception, 'flow.extend appended inconsistent flow'
+        raise Exception('flow.extend appended inconsistent flow')
     except ValueError:
         assert_equal(len(flow), length)
     # insert
@@ -176,7 +178,7 @@ def testFlow_container_listmethods():
     try:
         newnode = BogusNode(output_dim=11)
         flow.insert(2, newnode)
-        raise Exception, 'flow.insert inserted inconsistent node'
+        raise Exception('flow.insert inserted inconsistent node')
     except ValueError:
         assert_equal(len(flow), length)
     # pop
@@ -194,7 +196,7 @@ def testFlow_container_listmethods():
     flow._check_nodes_consistency(flow.flow)
     try:
         nottobepopped = flow.pop(4)
-        raise Exception, 'flow.pop left inconsistent flow'
+        raise Exception('flow.pop left inconsistent flow')
     except ValueError:
         assert_equal(len(flow), length)
 
@@ -276,7 +278,7 @@ def testCheckpointFunction():
                                   node_class = BogusNodeTrainable)
     flow.train(inp, cfunc)
     #
-    for i in xrange(len(flow)):
+    for i in range(len(flow)):
         assert flow[i].__class__==cfunc.classes[i], 'Wrong class collected'
 
 def testCrashRecovery():
@@ -284,7 +286,7 @@ def testCrashRecovery():
     flow.set_crash_recovery(1)
     try:
         flow.train(mdp.numx.zeros((1,2), 'd'))
-    except Exception, e:
+    except Exception as e:
         assert isinstance(e,mdp.FlowExceptionCR)
         with open(e.filename, 'rb') as fl:
             pic_flow = pickle.load(fl)
@@ -293,24 +295,28 @@ def testCrashRecovery():
     flow.set_crash_recovery(0)
     try:
         flow.execute([None])
-    except Exception, e:
+    except Exception as e:
         assert isinstance(e,mdp.FlowExceptionCR)
         assert not hasattr(e,'filename')
 
 def testCrashRecoveryException():
     a = 3
     try:
-        raise mdp.CrashRecoveryException, \
-              ('bogus errstr',a,StandardError())
-    except mdp.CrashRecoveryException, e:
+        raise mdp.CrashRecoveryException('bogus errstr', a, Exception())
+    except mdp.CrashRecoveryException as e:
         filename1 = e.dump()
-        filename2 = e.dump(os.path.join(tempfile.gettempdir(),'removeme'))
-        assert isinstance(e.parent_exception, StandardError)
+        filename2 = e.dump(tempfile.mkstemp(prefix='MDP_',
+                                            dir=py.test.mdp_tempdirname)[1])
+        assert isinstance(e.parent_exception, Exception)
 
-    for fname in [filename1,filename2]:
-        with open(fname, 'rb') as fl:
-            obj = pickle.load(fl)
-        os.remove(fname)
+    for fname in filename1, filename2:
+        fl = open(fname, 'rb')
+        obj = pickle.load(fl)
+        fl.close()
+        try:
+            os.remove(fname)
+        except Exception:
+            pass
         assert obj == a
 
 def testMultiplePhases():
@@ -319,7 +325,7 @@ def testMultiplePhases():
     flow.train(mdp.numx.zeros((1,2), 'd'))
     assert flow[0].visited == [1,2,3,4]
     # try to use an iterable to train it, check for rewinds
-    class TestIterable:
+    class TestIterable(object):
         def __init__(self):
             self.used = 0
         def __iter__(self):
